function syn2surfaceplot_ometiff( filename, colors, viewangles, alphaval )
% IMG2PLOT Helper method that displays images generated by CellOrganizer
%
% List Of Input Arguments   Descriptions
% -----------------------   ------------
% filename                  an ometiff image
% colors                    a cell array containing a list of valid colors as defined by Matlab
%                           http://www.mathworks.com/help/techdoc/ref/colorspec.html
% viewangles                a vector defining the viewing angle as defined by Matlab
%                           http://www.mathworks.com/help/techdoc/ref/view.html
% alphaval                  a value that determines the transparency of an object
%                           http://www.mathworks.com/help/techdoc/ref/alpha.html

% Xin Lu
%
% Copyright (C) 2012-2017 Murphy Lab
% Lane Center for Computational Biology
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu

% Author: Ivan E. Cao-Berg (icaoberg@scs.cmu.edu)
% July 21, 2012 R.F. Murphy Added arguments to control colors, viewangles,
% alpha
% August 10, 2012 I. Cao-Berg Updated docs and renamed method

%step 1: check input arguments
if ~isa( filename, 'char' )
    error( 'Input argument directory must be a string' )
end

if ~exist( filename )
    error( 'Input argument file must exist' )
end

if ~exist( 'colors', 'var')
    colors = {'yellow','blue','green','magenta','cyan','red'};
end

if ~exist( 'viewangles', 'var')
    viewangles = [50,-15];
end

if ~exist( 'alphaval', 'var')
    alphaval = 0.1;
end

%%%read ometiff
try
    reader = bfGetReader( filename );
catch err
    disp('Failed to get image reader');
    getReport( err )
    return
end
try
    omeMeta = reader.getMetadataStore();
catch err
    disp('Failed to get metadata store');
    getReport( err )
    return
end
try
    size_x = omeMeta.getPixelsSizeX(0).getValue();
    size_y = omeMeta.getPixelsSizeY(0).getValue();
    size_z = omeMeta.getPixelsSizeZ(0).getValue();
    size_c = omeMeta.getPixelsSizeC(0).getValue();
catch err
    disp('Failed to extract image information');
    getReport( err )
    return
end
%%%%
try
    clf
    for i=1:1:size_c
        img = OME_loadchannel(filename,i);
        p = patch( isosurface(img) );
        set( p, 'FaceColor', colors{mod(i-1,length(colors))+1}, 'EdgeColor', 'none' );
        view(viewangles(1),viewangles(2));
        hold on
    end
    
    alpha(alphaval);
    daspect([1 1 1]);
    axis off
    hold off
catch
    warning('Unable to display images.')
end
