import os
from re import A 
import torch
import torch.nn as nn

from R2UNet import R2U_Net as Generator
import importlib
import numpy as np

import scipy.io as sio
from tifffile import imread, imsave
import pandas as pd
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--models", action="store", type=str, nargs="+", help="list of paths cotaining trained models")
parser.add_argument("--retrainedModel", type=str, help='the model to be retrained')
parser.add_argument('--nn_module', default='vox2vox', help='name of neural network module: vox2vox or unet')
args = parser.parse_args()


structs = args.models



for struct in structs:
    if not os.path.isdir("vox2vox/generatedTrainingImages"):
        os.mkdir(r"vox2vox/generatedTrainingImages")
    if not os.path.isdir("vox2vox/generatedTrainingImages/{}".format(struct)):
        os.mkdir(r"vox2vox/generatedTrainingImages/{}".format(struct))
        for i in range(60):
            if not os.path.isdir(r"vox2vox/generatedTrainingImages/{}/{:02d}".format(struct, i)):
                os.mkdir(r"vox2vox/generatedTrainingImages/{}/{:02d}".format(struct, i))

train = True

for struct1 in structs:
    state_dict = torch.load(os.path.join(struct1, "model.p").format(struct1))
    if args.nn_module == "vox2vox":
        G = Generator(1)
    elif args.nn_module == "unet":
        G = importlib.import_module("fnet.nn_modules." + "fnet_nn_3d").Net()
    else: ValueError("unknown model")


    G.load_state_dict(state_dict['nn_G_state'])
    G.cuda()
    with torch.no_grad():
        for struct2 in structs:
            if "membrane_caax" in struct2:
                if train:
                    a = 38
                else:
                    a = 13
            else:
                if train:
                    a = 60
                else:
                    a = 20
            for i in range(0, a):
                torch.cuda.empty_cache()
                if train:
                    input_ = imread(r"imageDataResized/{0}/train/{0}/{1:02d}/signal.tiff".format(struct2, i)).astype("f")
                else:
                    input_ = imread(r"imageDataResized/{0}/test/{0}/{1:02d}/signal.tiff".format(struct2, i)).astype("f")

                input_ = input_.reshape(1, input_.shape[0], input_.shape[1], input_.shape[2], input_.shape[3])
                in_ = torch.from_numpy(input_).type(torch.float32).cuda()

                out_ = G(in_)
                out_ = out_.detach().cpu().numpy()[0, 0]
                imsave(r"vox2vox/generatedTrainingImages/{0}/{1:02d}/{} prediction.tiff".format(struct2, i, struct1), out_)


if "membrane_caax" in args.retrainedModel:
    a = 38
else: a = 60
for i in range(a):
    temp = []
    for s in args.models:
        if s != args.retrainedModel:
            temp.append(imread(r"vox2vox/generatedTrainingImages/{0}/{1:02d}/{} prediction.tiff".format(args.retrainedModel, i, s)))
    temp = np.array(temp)
    temp = np.max(temp, axis=0)
    temp = temp.reshape((1, temp.shape[0], temp.shape[1], temp.shape[2]))
    imsave(r"vox2vox/generatedTrainingImages/{0}/{1:02d}/max.tiff".format(args.retrainedModel, i), temp)

